<?php

namespace Survey\Gang\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Survey\Gang\Http\Requests\GangRequest;
use Survey\Gang\Interfaces\GangRepositoryInterface;
use Survey\Gang\Models\Gang;

/**
 * Resource controller class for gang.
 */
class GangResourceController extends BaseController
{

    /**
     * Initialize gang resource controller.
     *
     * @param type GangRepositoryInterface $gang
     *
     * @return null
     */
    public function __construct(GangRepositoryInterface $gang)
    {
        parent::__construct();
        $this->repository = $gang;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Gang\Repositories\Criteria\GangResourceCriteria::class);
    }

    /**
     * Display a list of gang.
     *
     * @return Response
     */
    public function index(GangRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Survey\Gang\Repositories\Presenter\GangPresenter::class)
                ->$function();
        }

        $gangs = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('gang::gang.names'))
            ->view('gang::gang.index', true)
            ->data(compact('gangs', 'view'))
            ->output();
    }

    /**
     * Display gang.
     *
     * @param Request $request
     * @param Model   $gang
     *
     * @return Response
     */
    public function show(GangRequest $request, Gang $gang)
    {

        if ($gang->exists) {
            $view = 'gang::gang.show';
        } else {
            $view = 'gang::gang.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('gang::gang.name'))
            ->data(compact('gang'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new gang.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(GangRequest $request)
    {

        $gang = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('gang::gang.name')) 
            ->view('gang::gang.create', true) 
            ->data(compact('gang'))
            ->output();
    }

    /**
     * Create new gang.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(GangRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $gang                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('gang::gang.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('gang/gang/' . $gang->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/gang/gang'))
                ->redirect();
        }

    }

    /**
     * Show gang for editing.
     *
     * @param Request $request
     * @param Model   $gang
     *
     * @return Response
     */
    public function edit(GangRequest $request, Gang $gang)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('gang::gang.name'))
            ->view('gang::gang.edit', true)
            ->data(compact('gang'))
            ->output();
    }

    /**
     * Update the gang.
     *
     * @param Request $request
     * @param Model   $gang
     *
     * @return Response
     */
    public function update(GangRequest $request, Gang $gang)
    {
        try {
            $attributes = $request->all();

            $gang->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('gang::gang.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('gang/gang/' . $gang->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('gang/gang/' . $gang->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the gang.
     *
     * @param Model   $gang
     *
     * @return Response
     */
    public function destroy(GangRequest $request, Gang $gang)
    {
        try {

            $gang->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('gang::gang.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('gang/gang/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('gang/gang/' . $gang->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple gang.
     *
     * @param Model   $gang
     *
     * @return Response
     */
    public function delete(GangRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('gang::gang.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('gang/gang'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/gang/gang'))
                ->redirect();
        }

    }

    /**
     * Restore deleted gangs.
     *
     * @param Model   $gang
     *
     * @return Response
     */
    public function restore(GangRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('gang::gang.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/gang/gang'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/gang/gang/'))
                ->redirect();
        }

    }

}
